/* CSS declarations for the "thumbnail" plugin */

.thumbnail {
    position: absolute;
    bottom: 5px;
    right: 5px;
    background-color: white;
    outline: 2px solid rgba(0,0,0,0.3);
    z-index: 100;
    overflow: hidden;
    opacity: 0.8;
}

.thumbnail:hover {
    opacity: 1;
}

.thumbnail .inner {
    width: 100%;
    height: 100%;
}
    
.thumbnail .inner .indicator {
    border: 1px solid black;
    position: absolute;
    outline: 3px solid rgba(100,100,100,0.5);
    pointer-events: none;
}

div.thumbnail-toggle {
    display: table-cell;
    position: absolute;
    width: 20px;
    height: 20px;
    right: 5px;
    bottom: 5px;
    z-index: 100;
    margin: 0;    
    border: 1px solid black;
    text-align: center;
    background-color: white;
}
